//// I2S configuration and transaction
/*----------------------------------------------------------------------------
 * Name:    I2S.c
 * Purpose: Freescale MK60X512VMD100 I2S Audio using TWR-AUDIO SGTL5000
 *----------------------------------------------------------------------------
//// CODE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
//// WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
//// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
//// BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
//// DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
//// PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
//// BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
//// ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
//// DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
//// INFRINGEMENT.
 *----------------------------------------------------------------------------*/

#include <MK60F12.H>
#include "I2S.h"


////////////////////////////////////////////////////////////////////
// Initialize I2S interface
////////////////////////////////////////////////////////////////////
void I2S0_Config(void) {

	SIM->SCGC5    	|= (1UL <<  13) | (1UL << 11); 		/// Enable Clock to Port E and C	
	SIM->SCGC6			|= (1UL << 15);										/// Enable Clock to I2S0/SAI
	
	PORTE->PCR[6] 	= (1UL << 10);					/// mux as alt4 (I2S0_MCLK)	---------------A21
	PORTE->PCR[12] 	= (1UL << 10);					/// mux as alt4 (I2S0_TX_BCLK) S_CLK --------A22		
// 	PORTE->PCR[11] 	= (1UL << 10);					/// mux as alt4 (I2S0_TX_FS) LRCLK --------B21	for k60n512 twr-board
	PORTC->PCR[2]		=	PORT_PCR_MUX(6); 			/// mux as alt6 for (I2S0_TX_FS) LRCLK ------A23	for k60F120 twr-board
	PORTE->PCR[7] 	= (1UL << 10);					/// mux as alt4 (I2S0_RXD) ------------------A24	
// 	PORTE->PCR[10] 	= (1UL << 10);					/// mux as alt4 (I2S0_TXD) ----------------A9   for k60n512 twr-board
	PORTC->PCR[1]		=	PORT_PCR_MUX(6); 			/// mux as alt6 for I2S0_TXD0 ---------------A25	for k60F120 twr-board
	PORTC->PCR[0]		=	PORT_PCR_MUX(6); 			/// mux as alt6 for I2S0_TXD1 ---------------A58 
	
	/// Transmitter setup
	I2S0->TCR1 			= I2S_TCR1_TFW((4));
	/// For using TX BLCK and FS CLK in synch mode, TX must be Asynch and Rx synch
	I2S0->TCR2 			= I2S_TCR2_SYNC(0) | I2S_TCR2_BCP_MASK;
	I2S0->MCR 			&= ~I2S_MCR_MOE_MASK;

	/// Transmit data channel 1 enabled
	I2S0->TCR3 			= I2S_TCR3_WDFL(0) | I2S_TCR3_TCE(1);
	
	/// 2 slot on a frame, Fsynch of 32 clocks
	I2S0->TCR4 			= I2S_TCR4_FRSZ(1) | I2S_TCR4_FSE_MASK | I2S_TCR4_SYWD(31) | I2S_TCR4_MF_MASK | I2S_TCR4_FSP_MASK;
	I2S0->TCR4 			&= ~(I2S_TCR4_FSD_MASK);

	/// 24-bit first and following words
	I2S0->TCR5 			= I2S_TCR5_WNW(31) | I2S_TCR5_W0W(31) | I2S_TCR5_FBT(31);
	/// Reset FIFO  
	I2S0->TCSR 			|=  I2S_TCSR_FR_MASK; 
	
	
	/// Reciever setup
	I2S0->RCR1 			= I2S_RCR1_RFW((4));
	I2S0->RCR2 			= I2S_RCR2_SYNC(1);
	I2S0->RCR3 			= I2S_RCR3_WDFL(0) | I2S_RCR3_RCE(1);

	I2S0->RCR4 			=  I2S_RCR4_FRSZ(1) | I2S_RCR4_FSE_MASK | I2S_RCR4_SYWD(31) | I2S_RCR4_MF_MASK | I2S_RCR4_FSP_MASK;
	I2S0->RCR4 			&= ~(I2S_RCR4_FSD_MASK);
	I2S0->RCR5 			= I2S_RCR5_WNW(31) | I2S_RCR5_W0W(31)|I2S_RCR5_FBT(31);
	I2S0->RCSR 			|= I2S_RCSR_FR_MASK; 
		

}

///////////////////////////////////////////////////////////////////////
/// 	Recieve I2S LT and RT channel data and tx LT and RT channels
///		pass pointers for right and left audio words
///////////////////////////////////////////////////////////////////////
void I2S_RxTx(int32_t* n32_rt_audio_in, int32_t* n32_lt_audio_in, int32_t n32_rt_audio_out, int32_t n32_lt_audio_out)	{

		I2S0->TDR[0] = n32_rt_audio_out & 0xFFFFFF00; 	
		I2S0->TCSR 			|= I2S_TCSR_TE_MASK;
		I2S0->RCSR 			|= I2S_RCSR_RE_MASK;			
		while(!(I2S0-> RCSR & I2S_RCSR_FRF_MASK)){}				
		*n32_lt_audio_in = I2S0->RDR[0];	

		I2S0->TDR[0] = n32_lt_audio_out & 0xFFFFFF00; 		
	 	I2S0->TCSR 			|= I2S_TCSR_TE_MASK;	
		I2S0->RCSR 			|= I2S_RCSR_RE_MASK;			
		while(!(I2S0-> RCSR & I2S_RCSR_FRF_MASK)){}				
		*n32_rt_audio_in = I2S0->RDR[0];
	
}



