//// SGTL5000 Audio Codec Init and other Functions
/*----------------------------------------------------------------------------
 * Name:    SGTL5000.c
 * Purpose: Freescale MK60X512VMD100 I2S Audio using TWR-AUDIO SGTL5000
 *----------------------------------------------------------------------------
//// CODE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
//// WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
//// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
//// BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
//// DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
//// PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
//// BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
//// ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
//// DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
//// INFRINGEMENT.
 *----------------------------------------------------------------------------*/
 
#include <MK60F12.H>
#include "SGTL5000.h"
#include "I2C.h" 




//////////////////////////////////////////////////////////////////////
/// I2C Writes to Configure the SGTL5000 Audio Codec
//////////////////////////////////////////////////////////////////////
void SGTL5000_I2C_Write(uint16_t u16RegisterAddress, uint16_t u16Data)	{
		int n = 0;

	
		///	Start I2C
		I2C0->C1    |= I2C_C1_TX_MASK;								/// select tx mode
		I2C0->C1    |= I2C_C1_MST_MASK;   						/// selsect master mode
	
		/// Device Address
		I2C0->D 		= (unsigned char)SGTL5000_I2C_ADDRESS << 1;					/// write device address  
		while((I2C0->S & I2C_S_IICIF_MASK)==0) {} 
    I2C0->S |= I2C_S_IICIF_MASK;
			
			
		/// Reg Address MSByte
		I2C0->D = (unsigned char)((u16RegisterAddress >> 8) & 0xFF);		/// write reg address
		while((I2C0->S & I2C_S_IICIF_MASK)==0) {} 
		I2C0->S |= I2C_S_IICIF_MASK;

		/// Reg Address MSByte
		I2C0->D = (unsigned char)(u16RegisterAddress & 0x00FF);					/// write reg address
		while((I2C0->S & I2C_S_IICIF_MASK)==0) {} 
		I2C0->S |= I2C_S_IICIF_MASK;				
					
		/// Reg Data MSByte
		I2C0->D = (unsigned char)((u16Data >> 8) & 0xFF);								/// write reg data
		while((I2C0->S & I2C_S_IICIF_MASK)==0) {} 
		I2C0->S |= I2C_S_IICIF_MASK;
			
		/// Reg Data	LSByte
		I2C0->D = (unsigned char)(u16Data & 0x00FF);										/// write reg data
		while((I2C0->S & I2C_S_IICIF_MASK)==0) {} 
		I2C0->S |= I2C_S_IICIF_MASK;	
		
		/// Stop I2C
		I2C0->C1  		&= ~I2C_C1_MST_MASK;						/// Diable master mode
		I2C0->C1  		&= ~I2C_C1_TX_MASK;							/// Disable tx mode

		/// Brief delay after txmit
		for(n=0; n<300; n++)
			{
 				__nop();
			}

} 
 
 
//////////////////////////////////////////////////////////////////////
/// Initialize the SGTL5000 Audio Codec
//////////////////////////////////////////////////////////////////////
void SGTL5000_Init( void)	{
	
	/// straight from SGTL5000 Ref Man
	
	/// CHIP_LINREG_CTRL 0x0026
	SGTL5000_I2C_Write(CHIP_LINREG_CTRL, 0x0008);		/// Configure VDDD level to 1.2V (bits 3:0)
	/// CHIP_ANA_POWER 0x0030		
	SGTL5000_I2C_Write(CHIP_ANA_POWER, 0x7260);			/// Power up internal linear regulator (Set bit 9)
	/// CHIP_LINREG_CTRL 0x0026		
	SGTL5000_I2C_Write(CHIP_LINREG_CTRL, (0x0008 | 0x0060));		/// Configure the charge pump to use the VDDIO rail (set bit 5 and bit 6)
																									///------ Reference Voltage and Bias Current Configuration----------
																									/// Set ground, ADC, DAC reference voltage (bits 8:4). The value should 
																									/// be set to VDDA/2. This example assumes VDDA = 3.3V. VDDA/2 = 1.65V.
	/// CHIP_REF_CTRL	0x0028												/// The bias current should be set to 50% of the nominal value (bits 3:1)
	SGTL5000_I2C_Write(CHIP_REF_CTRL, 0x01FE);			/// might try changing least sig byte to allow more bias current
																									/// Set LINEOUT reference voltage to VDDIO/2 (1.65V) (bits 5:0) and bias current 
																									/// (bits 11:8) to the recommended value of 0.36mA for 10kOhm load with 1nF 
	/// CHIP_LINE_OUT_CTRL 0x002C										/// capacitance
	SGTL5000_I2C_Write(CHIP_LINE_OUT_CTRL, 0x0322);
	/// CHIP_REF_CTRL	0x0028												///----------------Other Analog Block Configurations------------------
	SGTL5000_I2C_Write(CHIP_REF_CTRL, 0x01F0 | 0x0001);		/// Configure slow ramp up rate to minimize pop (bit 0)
																			//was 0x01FE						/// Enable short detect mode for headphone left/right 
																									/// and center channel and set short detect current trip level
	/// CHIP_SHORT_CTRL 0x003C											/// to 75mA	
	SGTL5000_I2C_Write(CHIP_SHORT_CTRL, 0x1106);
	/// CHIP_ANA_CTRL 0x0024
	SGTL5000_I2C_Write(CHIP_ANA_CTRL, 0x0133);	  	/// Enable Zero-cross detect if needed for HP_OUT (bit 5) and ADC (bit 1)
	/// CHIP_ANA_POWER	0x0030											///----------------Power up Inputs/Outputs/Digital Blocks-------------
	SGTL5000_I2C_Write(CHIP_ANA_POWER,  0x7260 | 0x6AFF);		/// Power up LINEOUT, HP, ADC, DAC  
																									/// Power up desired digital blocks
																									/// I2S_IN (bit 0), I2S_OUT (bit 1), DAP (bit 4), DAC (bit 5), 
///// CHIP_DIG_POWER	0x0002											/// ADC (bit 6) are powered on
// 	SGTL5000_I2C_Write(CHIP_DIG_POWER, 0x0073 & ~(1UL << 4));
	SGTL5000_I2C_Write(CHIP_DIG_POWER, 0x0073);	
																									///--------------------Set LINEOUT Volume Level-----------------------
																									/// Set the LINEOUT volume level based on voltage reference (VAG)
																									/// values using this formula
																									/// Value = (int)(40*log(VAG_VAL/LO_VAGCNTRL) + 15)
																									/// Assuming VAG_VAL and LO_VAGCNTRL is set to 0.9V and 1.65V respectively, the 
																									/// left LO volume (bits 12:8) and right LO volume (bits 4:0) value should be set 
	/// CHIP_LINE_OUT_VOL	0x002E										/// to 5
	SGTL5000_I2C_Write(CHIP_LINE_OUT_VOL, 0x0505);
																									/// Configure SYS_FS clock to 48kHz 
	/// CHIP_CLK_CTRL	0x0004												/// Configure MCLK_FREQ to 256*Fs
	SGTL5000_I2C_Write(CHIP_CLK_CTRL, 0x000A);  		
																									/// Configure the I2S clocks in master mode
	/// CHIP_I2S_CTRL	0x0006												/// NOTE: I2S LRCLK and data length 24 bits, word starts 1 bclk after lrclk transition
	SGTL5000_I2C_Write(CHIP_I2S_CTRL, 0x0090);	
	/// CHIP_ANA_POWER 0x0030												/// Power up the PLL
// 	SGTL5000_I2C_Write(CHIP_ANA_POWER, 0x7260 | 0x6AFF | 0x0500);
	
	/// CHIP_ANA_POWER 0x0030												/// Power up the PLL
	SGTL5000_I2C_Write(CHIP_ANA_POWER, 0x77FF);	
	
	
																									/// NOTE: This step is required only when the external SYS_MCLK
																									/// is above 17MHz. In this case the external SYS_MCLK clock 
	/// CHIP_CLK_TOP_CTRL	0x0034 										/// must be divided by 2
	SGTL5000_I2C_Write(CHIP_CLK_TOP_CTRL, 0x0008);
																									/// Set the PLL dividers
																									/// Int_Divisor = floor(PLL_Output_Freq/Sys_MCLK_Input_Freq)
	/// CHIP_PLL_CTRL	0x0032												/// Frac_Divisor = ((PLL_Output_Freq/Sys_MCLK_Input_Freq) - Int_Divisor)*2048
	SGTL5000_I2C_Write(CHIP_PLL_CTRL, 0x8000);
///// DAP_CONTROL 0x0100
	SGTL5000_I2C_Write(DAP_CONTROL, 0x0011);				/// Enable the DAP
	/// CHIP_ANA_CTRL	0x0024												/// Route LINEIN -> HP_OUT 0x0066   Route DAC -> HP_OUT 0x0026
	SGTL5000_I2C_Write(CHIP_ANA_CTRL, 0x0026 );   	/// unmute lineout, headphone, adc.  & ~(1UL << 5) & ~(1UL << 1)
// 	/// CHIP_SSS_CTRL 0x000A
// 	SGTL5000_I2C_Write(CHIP_SSS_CTRL, 0x0150);			/// select I2S_IN -> DAC and ADC -> I2S_DOUT 0x0150, ADC->DAC 0x0140

///// CHIP_SSS_CTRL 0x000A
	SGTL5000_I2C_Write(CHIP_SSS_CTRL, 0x0170);			/// select I2S_IN -> DAP and ADC -> I2S_DOUT 0x0170, DAP->DAC 0x0170

	/// CHIP_ADCDAC_CTRL 0x000E
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x0000);	

}

//////////////////////////////////////////////////////////////////////
/// SGTL5000 Audio Codec bypass: true for bypass, false for LINE_IN->ADC->I2S_DOUT->I2S_IN->DAC->HP
//////////////////////////////////////////////////////////////////////
void SGTL5000_Bypass(int bypass)	{
	
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x000C);	/// Mute DAC 
	
	if(bypass)
			SGTL5000_I2C_Write(CHIP_ANA_CTRL, 0x0066 );
	else
			SGTL5000_I2C_Write(CHIP_ANA_CTRL, 0x0026 );
	
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x0000);	/// UnMute DAC 
	
}

//////////////////////////////////////////////////////////////////////
/// SGTL5000 Audio Codec mute, true for mute flase for unmute
//////////////////////////////////////////////////////////////////////
void SGTL5000_mute(int mute)	{
	
	if(mute)
			SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x000C);	/// Mute DAC 
	else
			SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x0000);	/// UnMute DAC 
		
}

//////////////////////////////////////////////////////////////////////
/// Change DAC Volume  changes in 0.5017 dB steps
//	0x3C = 0 dB, 0x3D = -0.5 dB, 0xF0 = -90 dB, 0xFC and greater = Muted
//////////////////////////////////////////////////////////////////////
void SGTL5000_DAC_Vol(uint8_t Vol_Rt, uint8_t Vol_Lt)	{
	
	uint16_t un16_Vol_RtLt = 0;
	
	if(Vol_Rt < 0x3C && Vol_Lt < 0x3C)
		un16_Vol_RtLt = 0x3C3C;
	else if(Vol_Rt < 0x3C)
		un16_Vol_RtLt = (0x3C00) | Vol_Lt;
	else if(Vol_Lt < 0x3C)
		un16_Vol_RtLt = (Vol_Rt << 8) | 0x3C;
	else
		un16_Vol_RtLt = (Vol_Rt << 8) | Vol_Lt;

	SGTL5000_I2C_Write(CHIP_DAC_VOL, un16_Vol_RtLt);	/// Change DAC Volume 
		
}

//////////////////////////////////////////////////////////////////////
/// Change Bass Enhance
//	HPF 0 = enable, 1 = bypass;  midCut 0 = 80Hz, 1 = 100 Hz,
//	2 = 125 Hz, 3 = 150 Hz, 4 = 175 Hz, 5 = 200 Hz, 6 = 225 Hz; 
// 	Bass 0 = disable, 1 = enable		
//////////////////////////////////////////////////////////////////////
void SGTL5000_DAP_Bass_En(int HPF, int midCut, int Bass)	{
	
	uint16_t HPF_Mc_Bass = (HPF << 8) | (midCut << 4) | (Bass << 0);	
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x000C);	/// Mute DAC 
	SGTL5000_I2C_Write(DAP_BASS_ENHANCE, HPF_Mc_Bass);	
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x0000);	/// UnMute DAC	
}

//////////////////////////////////////////////////////////////////////
/// Change Bass Enhance CONTROL
//	LR_LEVEL reset to 0x5,  0x00 = +6 dB, 0x3F = Least L/R Channel
//	BASS_LEVEL	reset 0x1f, 0x00 = mst harmonic boost, 0x7F = Least boost 
// 		
//////////////////////////////////////////////////////////////////////
void SGTL5000_DAP_Bass_En_CNTL(int LR_Level, int Bass_Level)	{
	
	uint16_t Bass_En_Ctrl = ((LR_Level << 8) | (Bass_Level << 0)) & 0x3F7F;
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x000C);	/// Mute DAC 
	SGTL5000_I2C_Write(DAP_BASS_ENHANCE_CTRL, Bass_En_Ctrl);	
	SGTL5000_I2C_Write(CHIP_ADCDAC_CTRL, 0x0000);	/// UnMute DAC	
}

//////////////////////////////////////////////////////////////////////
/// Change HP Volume   changes in 0.5 dB steps
//	0x00 = +12 dB, 0x01 = +11.5 dB, 0x18 = 0 dB, 0x7F = -51.5 dB
//////////////////////////////////////////////////////////////////////
void SGTL5000_HP_Vol(uint8_t Vol_Rt, uint8_t Vol_Lt)	{
	
	uint16_t un16_Vol_RtLt = 0;
	
	if(Vol_Rt >= 0x7F && Vol_Lt >= 0x7F)
		un16_Vol_RtLt = 0x7F7F;
	else if(Vol_Rt >= 0x7F)
		un16_Vol_RtLt = (0x7F00) | Vol_Lt;
	else if(Vol_Lt >= 0x7F)
		un16_Vol_RtLt = (Vol_Rt << 8) | 0x007F;
	else
		un16_Vol_RtLt = (Vol_Rt << 8) | Vol_Lt;

	SGTL5000_I2C_Write(CHIP_ANA_HP_CTRL, un16_Vol_RtLt);	/// Change HP Volume 
		
}

//////////////////////////////////////////////////////////////////////
/// Change Line Out Volume   changes in 0.5 dB steps reset to 0x4
//  for VDDA 3.3v, VAG_VAL 1.55v, VDDIO 3.3v, LO_VAGCNTRL 1.55v
//	40*log(VAG_VAL/LO_VAGCNTRL)+15 = 15 or 0xF
//	higher codes have more attenuation
//////////////////////////////////////////////////////////////////////
void SGTL5000_LO_Vol(uint8_t Vol_Rt, uint8_t Vol_Lt)	{
	
	uint16_t un16_Vol_RtLt = 0;
	
	if(Vol_Rt >= 0x1F && Vol_Lt >= 0x1F)
		un16_Vol_RtLt = 0x1F1F;
	else if(Vol_Rt >= 0x1F)
		un16_Vol_RtLt = (0x1F00) | Vol_Lt;
	else if(Vol_Lt >= 0x1F)
		un16_Vol_RtLt = (Vol_Rt << 8) | 0x001F;
	else
		un16_Vol_RtLt = (Vol_Rt << 8) | Vol_Lt;

	SGTL5000_I2C_Write(CHIP_LINE_OUT_VOL, un16_Vol_RtLt);		/// Change LO Volume 
		
}

//////////////////////////////////////////////////////////////////////
/// Change DAP Main Ch Volume    reset to 0x8000
//  0xFFFF = 200%,  0x8000 = 100%,  0x0000 = 0%
//////////////////////////////////////////////////////////////////////
void SGTL5000_DAP_Mch_Vol(uint16_t Vol)	{
		
	SGTL5000_I2C_Write(DAP_MAIN_CHAN, Vol);					/// Change DAP Main Channel Volume 
		
}

//////////////////////////////////////////////////////////////////////
/// Freescale Surr Width Control and select    reset to 0x04 and 0x0
//  0x0 least width, 0x7 most width; 0x1/2 disabled, 0x2 mono input,
//	0x3 Stereo input
//////////////////////////////////////////////////////////////////////
void SGTL5000_DAP_Surround(uint8_t Width, uint8_t Select)	{
		
	uint8_t Width_Crtl_Sel = ((Width << 4) | (Select << 0)) & 0x0073;
	SGTL5000_I2C_Write(DAP_SGTL_SURROUND, Width_Crtl_Sel);					/// Select Surround and width  
		
}

///////////////////////////////////////////////////////////////////////
///		Adjust ADC Volume  changes in 1.5 dB steps
//  	
//	0x3C = 0 dB, 0x3D = -0.5 dB, 0xF0 = -90 dB, 0xFC and greater = Muted
///////////////////////////////////////////////////////////////////////
void SGTL5000_ADC_Vol(uint8_t ADC_Range_Reduction, uint8_t Vol_Rt, uint8_t Vol_Lt)	{
	
	uint16_t un16_Vol_RtLt = 0;
	
	if(Vol_Rt > 0xF && Vol_Lt > 0xF)
		un16_Vol_RtLt = 0x00FF | ((ADC_Range_Reduction & 0x01) << 8);
	else if(Vol_Rt > 0xF)
		un16_Vol_RtLt = (0x00F0) | Vol_Lt | ((ADC_Range_Reduction & 0x01) << 8);
	else if(Vol_Lt > 0xF)
		un16_Vol_RtLt = 0x000F | (Vol_Rt << 4) | ((ADC_Range_Reduction & 0x01) << 8);
	else
		un16_Vol_RtLt = (Vol_Rt << 4) | Vol_Lt | ((ADC_Range_Reduction & 0x01) << 8);
	
	/// CHIP_ANA_ADC_CTRL 0x0020 , 0x0133 defalut	  
	SGTL5000_I2C_Write(CHIP_ANA_ADC_CTRL, un16_Vol_RtLt);	/// Change DAC Volume 
	
}





 
